@@ -9,16 +9,16 @@ fn main() -> Result<()> {
     let mut config = Config::new();
     config.consume_fuel(true);
     let engine = Engine::new(&config)?;
-    let store = Store::new(&engine);
+    let mut store = Store::new(&engine, ());
     store.add_fuel(10_000)?;
     let module = Module::from_file(store.engine(), "examples/fuel.wat")?;
-    let instance = Instance::new(&store, &module, &[])?;
+    let instance = Instance::new(&mut store, &module, &[])?;
 
     // Invoke `fibonacci` export with higher and higher numbers until we exhaust our fuel.
-    let fibonacci = instance.get_typed_func::<i32, i32>("fibonacci")?;
+    let fibonacci = instance.get_typed_func::<i32, i32, _>(&mut store, "fibonacci")?;
     for n in 1.. {
         let fuel_before = store.fuel_consumed().unwrap();
-        let output = match fibonacci.call(n) {
+        let output = match fibonacci.call(&mut store, n) {
             Ok(v) => v,
             Err(_) => {
                 println!("Exhausted fuel computing fib({})", n);
