@@ -40,16 +40,9 @@ const CRATES_TO_PUBLISH: &[&str] = &[
     "cranelift-interpreter",
     "cranelift",
     "cranelift-jit",
-    // wig/wiggle
+    // wiggle
     "wiggle-generate",
     "wiggle-macro",
-    "wiggle",
-    "wiggle-borrow",
-    "wasmtime-wiggle-macro",
-    // wasi-common
-    "wasi-common",
-    "wasi-cap-std-sync",
-    "wasi-tokio",
     // wasmtime
     "lightbeam",
     "wasmtime-fiber",
@@ -63,7 +56,12 @@ const CRATES_TO_PUBLISH: &[&str] = &[
     "wasmtime-jit",
     "wasmtime-cache",
     "wasmtime",
-    "wasmtime-wiggle",
+    // wasi-common/wiggle
+    "wiggle",
+    "wasi-common",
+    "wasi-cap-std-sync",
+    "wasi-tokio",
+    // other mic wasmtime crates
     "wasmtime-wasi",
     "wasmtime-wasi-nn",
     "wasmtime-wasi-crypto",
@@ -332,6 +330,7 @@ fn verify(crates: &[Crate]) {
         if krate.name.contains("lightbeam")
             || krate.name == "witx"
             || krate.name.contains("wasi-nn")
+            || krate.name.contains("peepmatic")
         {
             cmd.arg("--no-verify");
         }
