@@ -20,11 +20,12 @@ mod module;
 mod module_linking;
 mod module_serialize;
 mod name;
+mod native_hooks;
 mod pooling_allocator;
 mod stack_overflow;
+mod store;
 mod table;
 mod traps;
-mod use_after_drop;
 mod wast;
 
 // TODO(#1886): Cranelift only supports reference types on x64.
@@ -37,7 +38,7 @@ mod gc;
 #[cfg(target_arch = "x86_64")]
 pub(crate) fn ref_types_module(
     source: &str,
-) -> anyhow::Result<(wasmtime::Store, wasmtime::Module)> {
+) -> anyhow::Result<(wasmtime::Store<()>, wasmtime::Module)> {
     use wasmtime::*;
 
     let _ = env_logger::try_init();
@@ -46,7 +47,7 @@ pub(crate) fn ref_types_module(
     config.wasm_reference_types(true);
 
     let engine = Engine::new(&config)?;
-    let store = Store::new(&engine);
+    let store = Store::new(&engine, ());
 
     let module = Module::new(&engine, source)?;
 
